@@ -46,9 +46,10 @@ class Template(object):
         self.__check()
 
         if using_other:
+            stop_string = 'ssdlh,lhbzj,ddxss'
         # 如果使用他人的账户
-            command = "\n".join([work_dir, init_text, exec])
-            exec = f"ChangeUser -u {self.work_user} -p {self.work_password} -c \"{command}\""
+            command = "\n".join([work_dir, init_text, exec, f"echo {stop_string}"])
+            exec = f"ChangeUser -u {self.work_user} -p {self.work_password} -e \"{stop_string}\" -c \"{command}\""
             work_dir = ''
             init_text = ''
 
@@ -90,7 +91,7 @@ class Template(object):
     
     def __check(self):
         assert self.log_name.endswith(".log"), "log must end with .log"
-        assert self.bash_name.endswith(".sh"), "log must end with .sh"
+        assert self.bash_name.endswith(".sh"), "bash must end with .sh"
     
     def expand_permissions(self, _dir):
         # child = pexpect.spawn(f'cd {self.bash_dir}', encoding='utf-8')
