@@ -25,22 +25,39 @@ class MetricSaverBase(object):
                 setattr(self, l, None)
 
     @property
-    def best_acc(self):
+    def strict_best_acc(self):
+        """采用严格模式的acc，也就是取验证集上loss最小的测试集结果
+
+        Returns:
+            [type] -- [description]
+        """
         if len(self._test_acc) == 0:
             self._best_acc = 0
         else:
-            min_index = np.argmin(self._eva_acc)
+            min_index = np.argmin(self._eva_loss)
             self._best_acc = self._test_acc[min_index]
         return self._best_acc
+    
+    @property
+    def best_acc(self):
+        """采用非严格模式的acc，也就是只取验证集上的最大值
 
-
+        Returns:
+            [type] -- [description]
+        """
+        if len(self._eva_acc) == 0:
+            self._best_acc = 0
+        else:
+            # min_index = np.argmin(self._eva_acc)
+            self._best_acc = max(self._eva_acc)
+        return self._best_acc
     
     @property
     def mean_acc(self):
-        if len(self._test_acc) == 0:
-            self._test_acc = 0
+        if len(self._eva_acc) == 0:
+            self._mean_acc = 0
         else:
-            self._mean_acc = np.mean(self._test_acc)
+            self._mean_acc = np.mean(self._eva_acc)
         return self._mean_acc
     
     @property
@@ -134,9 +151,17 @@ class MetricSaverBase(object):
     def start(self):
         return
 
+    @property
+    def has_test(self):
+        return len(self._test_acc)
+
     def save(self, dir_name='.', prefix="", suffix=""):
-        file_name = prefix + "{:.4f}".format(self.best_acc) + suffix
+        if len(self._test_acc):
+            file_name = prefix + "{:.4f}-{:.4f}".format(self.strict_best_acc, self.best_acc) + suffix
+        else:
+            file_name = prefix + "{:.4f}".format(self.best_acc) + suffix
         dir_name = os.path.join(dir_name, file_name)
+
         if not os.path.exists(dir_name):
             os.makedirs(dir_name)
             # print(f"made new {dir_name} here.")
@@ -148,42 +173,96 @@ class MetricSaverBase(object):
             dill.dump(self._eva_acc, f)
         with open(f'{dir_name}/eva_loss.pkl', 'wb') as f:
             dill.dump(self._eva_loss, f)
-        with open(f'{dir_name}/test_acc.pkl', 'wb') as f:
-            dill.dump(self._test_acc, f)
-        with open(f'{dir_name}/test_loss.pkl', 'wb') as f:
-            dill.dump(self._test_loss, f)
-        ans = {'mean_acc': self.mean_acc,
-                'best_acc': self.best_acc}
-        with open('{}/{:.4f}.json'.format(dir_name, self.best_acc), 'w') as f:
-            json.dump(ans, f)
+        # 如果存在测试集，则同时保存两个结果
+        if len(self._test_acc):
+            with open(f'{dir_name}/test_acc.pkl', 'wb') as f:
+                dill.dump(self._test_acc, f)
+            with open(f'{dir_name}/test_loss.pkl', 'wb') as f:
+                dill.dump(self._test_loss, f)
+        if len(self._test_acc):
+            ans = {'strict_acc': self.strict_best_acc,
+                    'best_acc': self.best_acc,
+                    'mean_acc': self.mean_acc}
+            with open('{}/{:.4f}-{:.4f}.json'.format(dir_name, self.strict_best_acc, self.best_acc), 'w') as f:
+                json.dump(ans, f)
+        else:
+            ans = {'mean_acc': self.mean_acc,
+                    'best_acc': self.best_acc}
+            with open('{}/{:.4f}.json'.format(dir_name, self.best_acc), 'w') as f:
+                json.dump(ans, f)
     
-    def EarlyStopping(self, monitor='acc', warmup=50, min_delta=0.01, patience=20, mode='max'):
-        assert monitor in ['acc', 'loss'], "your monitor is not right"
-        assert mode in ['min', 'max'], "your stopping mode is not right"
-        assert isinstance(patience, int) and patience >= 2, "patience must be int and should bigger than two"
+    def EarlyStopping(self, monitor='acc', warmup=50, min_delta=0.01, patience=20, strict=None):
+        """[summary]
 
-        if monitor == 'acc':
-            if len(self._eva_acc) < warmup + patience:
-                return False
-            else:
-                check_ans = self._eva_acc[-patience:]
-        elif monitor == 'loss':
-            if len(self._eva_loss) < warmup + patience:
-                return False
-            else:
-                check_ans = self._eva_loss[-patience:]
+        Keyword Arguments:
+            monitor {str} -- [用哪个值观测是否早停] (default: {'acc'})
+            warmup {int} -- [从哪一轮开始观测早停] (default: {50})
+            min_delta {float} -- [变动的容忍值] (default: {0.01})
+            patience {int} -- [连续多少轮没有变动就早停] (default: {20})
+            strict {[type]} -- [description] (default: {None})
+
+        Returns:
+            bool - True: stop, False: continue
+        """
+        assert monitor in ['acc', 'loss'], "your monitor is not right"
+        assert isinstance(patience, int) and patience >= 2, "patience must be int and should bigger than two"
+        
+        if monitor == 'loss':
+            return self.ES_loss(warmup, min_delta, patience, strict)
+        elif monitor == 'acc':
+            return self.ES_acc(warmup, min_delta, patience, strict)
+
+    def ES_acc(self, warmup, min_delta, patience, strict):
+        """[summary]
+
+        Arguments:
+            strict {[type]} -- 是否在有test集合的情况下使用test集合作为早停的标准
+
+        Returns:
+            [type] -- [description]
+        """
+
+        if len(self._test_acc) and strict in [None, True]: # 在有test集合下默认使用test集合的acc来判断
+            check_ans = self._test_acc
+        else:
+            check_ans = self._eva_acc
+
+        if len(check_ans) < warmup + patience: # 判断是否满足了要求的开始早停的轮数
+            return False
+        else:
+            check_ans = check_ans[:patience]
+
+        # 获取patience列表中前半和后半的数
+        middle = patience // 2
+        front_ans = check_ans[:middle]
+        rear_ans = check_ans[middle:]
+        # 早停的逻辑是：前半结果的极值和后半结果的极值的差值小于min_delta
+        if (np.max(rear_ans) - np.min(front_ans)) <= min_delta:
+                return True
+        return False
+    
+    def ES_loss(self, warmup, min_delta, patience, strict):
+        if len(self._test_loss) and strict in [None, True]: # 在有test集合下默认使用test集合的acc来判断
+            check_ans = self._test_loss
+        else:
+            check_ans = self._eva_loss
+
+        if len(check_ans) < warmup + patience: # 判断是否满足了要求的开始早停的轮数
+            return False
+        else:
+            check_ans = check_ans[:patience]
         
         # 获取patience列表中前半和后半的数
         middle = patience // 2
         front_ans = check_ans[:middle]
         rear_ans = check_ans[middle:]
         # 早停的逻辑是：前半结果的极值和后半结果的极值的差值小于min_delta
-        if mode == 'max' and (np.max(rear_ans) - np.min(front_ans)) <= min_delta:
-                return True
-        elif mode == 'min' and (np.max(front_ans) - np.min(rear_ans)) <= min_delta:
+        if np.max(front_ans) - np.min(rear_ans) <= min_delta:
             return True
         return False
 
+    
+
 class TensorBoardSaver(MetricSaverBase):
     def __init__(self, log_dir='runs/', file_name=None, adding_attributes=None, tag='fold_0'):
         super(TensorBoardSaver, self).__init__(adding_attributes)
@@ -208,23 +287,11 @@ class TensorBoardSaver(MetricSaverBase):
             self.subp.kill()
         except AttributeError:
             pass
+    
 
     def add_record(self, epoch_train_acc, epoch_train_loss, epoch_eva_acc=None, epoch_eva_loss=None, epoch_test_acc=None, epoch_test_loss=None):
         flag = super().add_record(epoch_train_acc, epoch_train_loss, epoch_eva_acc, epoch_eva_loss, epoch_test_acc, epoch_test_loss)
         tag = self.tag
-        # self.writer.add_scalars("train_acc", {tag: epoch_train_acc}, len(self._train_acc))
-        # self.writer.add_scalars("train_loss", {tag: epoch_train_loss}, len(self._train_loss))
-        # if epoch_eva_acc is not None:
-        #     self.writer.add_scalars("eva_acc", {tag: epoch_eva_acc}, len(self._eva_acc))
-        # if epoch_eva_loss is not None:
-        #     self.writer.add_scalars("eva_loss", {tag: epoch_eva_loss}, len(self._eva_loss))
-
-        # self.writer.add_scalar("train_acc", epoch_train_acc, len(self._train_acc)-1)
-        # self.writer.add_scalar("train_loss", epoch_train_loss, len(self._train_loss)-1)
-        # if epoch_eva_acc is not None:
-        #     self.writer.add_scalar("eva_acc", epoch_eva_acc, len(self._eva_acc)-1)
-        # if epoch_eva_loss is not None:
-        #     self.writer.add_scalar("eva_loss", epoch_eva_loss, len(self._eva_loss)-1)
 
         self.writer.add_scalar(f"train_acc/train_acc_{tag}", epoch_train_acc, len(self._train_acc)-1)
         self.writer.add_scalar(f"train_loss/train_loss_{tag}", epoch_train_loss, len(self._train_loss)-1)
@@ -233,14 +300,15 @@ class TensorBoardSaver(MetricSaverBase):
         if epoch_eva_loss is not None:
             self.writer.add_scalar(f"eva_loss/eva_loss_{tag}", epoch_eva_loss, len(self._eva_loss)-1)
         if epoch_test_acc is not None:
-            self.writer.add_scalar(f"eva_acc/test_acc_{tag}", epoch_test_acc, len(self._test_acc)-1)
+            self.writer.add_scalar(f"test_acc/test_acc_{tag}", epoch_test_acc, len(self._test_acc)-1)
         if epoch_test_loss is not None:
-            self.writer.add_scalar(f"eva_loss/test_loss_{tag}", epoch_test_loss, len(self._test_loss)-1)
+            self.writer.add_scalar(f"test_loss/test_loss_{tag}", epoch_test_loss, len(self._test_loss)-1)
         return flag
     
-    def close(self):
+    def close(self, close_tb=False):
         self.writer.close()
-        self.close_server()
+        if close_tb:
+            self.close_server()
     
     def __del__(self):
         try:
@@ -250,11 +318,32 @@ class TensorBoardSaver(MetricSaverBase):
             pass
 
 class FoldMetricBase(object):
-    def __init__(self,  k_fold=10, saver=MetricSaverBase, dir_name='.', file_name='fold_test', timestamp=True, suffix=None, tb_server=False, adding_attributes=None):
+    """[summary]
+
+    Keyword Arguments:
+        k_fold {int} -- [description] (default: {10})
+        saver {[type]} -- [description] (default: {MetricSaverBase})
+        dir_name {str} -- [description] (default: {'.'})
+        file_name {str} -- [description] (default: {'fold_test'})
+        timestamp {bool} -- [description] (default: {True})
+        suffix {[type]} -- [description] (default: {None})
+        tb_server {bool} -- [description] (default: {False})
+        adding_attributes {[type]} -- [description] (default: {None})
+    """         
+    def __init__(self,  
+                k_fold=10, 
+                saver=MetricSaverBase, 
+                dir_name='.', 
+                file_name='fold_test', 
+                timestamp=True, 
+                suffix=None, 
+                tb_server=False, 
+                adding_attributes=None):
         # super(self, FoldMetricBase).__init__(adding_attributes)
         self._fold_metric_saver = []
         self._cur_k = 0
         self._fold_acc = []
+        self._strict_fold_acc = []
         self._k_fold = k_fold
 
         self.timestamp = time.strftime('%Y%m%d-%H_%M_%S', time.localtime(time.time()))
@@ -268,7 +357,9 @@ class FoldMetricBase(object):
             assert saver.__name__ in ['TensorBoardSaver', 'MetricSaverBase'], "your saver is not register in the FoldMetricBase, check the version or your spell"
             if saver.__name__ == 'TensorBoardSaver':
                 self.tb = tb_server
-                self._fold_metric_saver.append(saver(log_dir=os.path.join(dir_name, file_name, 'runs'), file_name=self.timestamp, tag=f'fold_{k}'))
+                self._fold_metric_saver.append(saver(log_dir=os.path.join(dir_name, file_name, 'runs'), 
+                                                    file_name=self.timestamp, 
+                                                    tag=f'fold_{k}'))
                 if k == 0:
                     self._fold_metric_saver[0].start(self.tb)
                     # self._fold_metric_saver[0].start_server()
@@ -283,7 +374,10 @@ class FoldMetricBase(object):
             _time = ""
 
         if suffix is None:
-            suffix = '{:.4f}'.format(self.mean_acc)
+            if len(self._strict_fold_acc):
+                suffix = '{:.4f}-{:.4f}'.format(self.strict_mean_acc, self.mean_acc)
+            else:
+                suffix = '{:.4f}'.format(self.mean_acc)
 
         dir_name = os.path.join(dir_name, file_name, file_name+'-'+suffix+'-'+_time)
         return dir_name
@@ -295,6 +389,13 @@ class FoldMetricBase(object):
         else:
             return np.mean(self._fold_acc)
     
+    @property
+    def strict_mean_acc(self):
+        if len(self._strict_fold_acc) == 0:
+            return 0.00
+        else:
+            return np.mean(self._strict_fold_acc)
+
     @property
     def cur_k(self):
         return min(len(self._fold_acc), self._cur_k)
@@ -305,6 +406,13 @@ class FoldMetricBase(object):
             return 0.00
         else:
             return np.std(self._fold_acc)
+
+    @property
+    def strict_std(self):
+        if len(self._strict_fold_acc) == 0:
+            return 0.00
+        else:
+            return np.std(self._strict_fold_acc)
     
     @property
     def cur_saver(self):
@@ -354,10 +462,15 @@ class FoldMetricBase(object):
         return self._fold_acc
 
     def next_fold(self):
-        self._fold_metric_saver[self._cur_k].close()
         self._fold_acc.append(self._fold_metric_saver[self._cur_k].best_acc)
+
+        if self._fold_metric_saver[self._cur_k].has_test:
+            self._strict_fold_acc.append(self._fold_metric_saver[self._cur_k].strict_best_acc)
+
         if self._cur_k + 1 >= self._k_fold:
+            self._fold_metric_saver[self._cur_k].close(close_tb=True)
             return
+        self._fold_metric_saver[self._cur_k].close()
         # 保存当前fold的acc
         # 指向当前fold的指针加1
         self._cur_k += 1
@@ -385,14 +498,27 @@ class FoldMetricBase(object):
         dir_name = self.gen_path(self.dir_name, self.file_name, self.timestamp, self.suffix)
         for i in range(self._k_fold):
             self._fold_metric_saver[i].save(dir_name, prefix=f'fold_{i}: ')
-        ans = {
-           'mean_acc': self.mean_acc,
-           'std': self.std,
-           'list': self._fold_acc
-        }
-        with open('{}/{:.4f}.json'.format(dir_name, self.mean_acc), 'w') as f:
-            json.dump(ans, f)
-        return 'save to {dir_name}'
+
+        if len(self._strict_fold_acc):
+            ans = {
+            'strict_mean_acc': self.strict_mean_acc,
+            'strict_std': self.strict_std,
+            'strict_list': self._strict_fold_acc,
+            'mean_acc': self.mean_acc,
+            'std': self.std,
+            'list': self._fold_acc
+            }
+            with open('{}/{:.4f}-{:.4f}.json'.format(dir_name, self.strict_mean_acc, self.mean_acc), 'w') as f:
+                json.dump(ans, f)
+        else:
+            ans = {
+            'mean_acc': self.mean_acc,
+            'std': self.std,
+            'list': self._fold_acc
+            }
+            with open('{}/{:.4f}.json'.format(dir_name, self.mean_acc), 'w') as f:
+                json.dump(ans, f)
+        return f'save to {self.dir_name}'
 
     def __repr__(self):
         return f"Fold:{self._cur_k}/{self._k_fold} ||" + self._fold_metric_saver[self._cur_k].__repr__() + " || fold_mean_acc:{:.4f}, fold_best_acc:{:.4f}".format(self.mean_acc, self.best_acc)
@@ -403,17 +529,22 @@ if __name__ == "__main__":
     fold = 10
     # b = FoldMetricBase(k_fold=fold, adding_attributes=['x'])
     # b = FoldMetricBase(k_fold=fold, saver=TensorBoardSaver, dir_name='test_fold', file_name='test_tensorboard', adding_attributes=['x'], tb_server=True)
-    b = FoldMetricBase(k_fold=fold, saver=TensorBoardSaver, dir_name='test_fold', file_name='test_tensorboard', adding_attributes=['x'], tb_server=False)
+    b = FoldMetricBase(k_fold=fold, 
+                       saver=TensorBoardSaver, 
+                       dir_name='test_fold', 
+                       file_name='test_tensorboard', 
+                       adding_attributes=['x'], 
+                       tb_server=False)
     for _ in range(fold):
         for i in range(100):
             _a, _b, _c, _d, _e, _f = np.random.rand(6)
             if b(_a, _b, _c, _d, _e, _f):
                 print(_, i, _d, _e)
             # time.sleep(0.1)
-            if b.EarlyStopping(warmup=50, patience=20, min_delta=0.001):
-                print(_, i, 'continue !!!')
-                break
-            time.sleep(1)
+            # if b.EarlyStopping(warmup=50, patience=20, min_delta=0.001):
+            #     print(_, i, 'continue !!!')
+            #     break
+            # time.sleep(1)
         b.next_fold()
     b.save()
 
